package com.lc.day;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Date;

/**
 * ClassName: MyClassLoaderTest
 *
 * @Author 爱扣钉-陈晨
 * @Create 2024/1/11 12:36
 * @Version 1.0
 */
public class MyClassLoaderTest {

    static class MyClassLoader extends ClassLoader {
        private String classPath;

        public MyClassLoader(String classPath) {
            this.classPath = classPath;
        }

        private byte[] loadByte(String name) throws IOException {
            name = name.replace("\\.","/");
            FileInputStream fis = new FileInputStream(classPath + "/" + name + ".class");

            int available = fis.available();

            byte[] data = new byte[available];

            fis.read(data);

            fis.close();

            return data;
        }

        protected Class<?> findClass(String name) throws ClassNotFoundException {
            try {
                byte[] data = loadByte(name);
                // defineClass将一个字节数组转为Class对象，这个字节数组是class文件读取后最终的字节数组。
                return defineClass(name, data, 0, data.length);
            } catch (IOException e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            }
        }
    }

    public static void main(String[] args) throws Exception {
        //初始化自定义类加载器，会先初始化父类ClassLoader，其中会把自定义类加载器的父加载器设置为应用程序类加载器AppClassLoader
        MyClassLoader myClassLoader = new MyClassLoader("D:/test");
        //D盘创建test/com/tuling/jvm 几级目录，将User类的复制类User1.class丢入该目录
        Class clazz=myClassLoader.loadClass("com.xxeg.jvm.User1");
        Object obj=clazz.newInstance();

        Method method = clazz.getDeclaredMethod("sout", null);

        method.invoke(obj, null);

        System.out.println(clazz.getClassLoader().getClass().getName());
    }
}
